CREATE OR REPLACE PROCEDURE VERIFICA_ERRO_CTB(
   P_NRLANCCTBTEMP IN LANCCONT.NRLANCCONT%TYPE,
   P_CDEMPRESA     IN EMPRESA.CDEMPRESA%TYPE)

IS
   V_COUNT FLOAT;
   V_RAT VARCHAR2(01);
   V_CONTRAT  VARCHAR2(01);
   V_CONTCOME VARCHAR2(01);
   V_IDSISTEMA VARCHAR2(03);
BEGIN
   /*
   ---------    CDIGOS DE ERRO DE ACORDO COM A TELA CTB10200 ----------
   001 - Total de crdito difere do total de dbito.
   002 - Data fora do perodo contbil.
   003 - Partida %s - Conta contbil no cadastrada.
   004 - Partida %s - Histrico padro no cadastrado.
   005 - Partida %s - Cdigo comercial no cadastrado.
   006 - Partida %s - Valor da partida zerado.
   007 - Partida %s - Valor de rateio zerado.
   008 - Partida %s - Filial de rateio no cadastrada.
   009 - Partida %s - Centro de custo no cadastrado para filial informada.
   010 - Partida %s - Soma dos rateios difere do valor da partida.
   011 - Partida %s - Conta contbil sem cdigo comercial informado.
   012 - Partida %s - Conta Contbil Gerencial sem rateio informado.
   013 - Partida %s - Centro de custo deve ser informado.
   014 - -- RESERVADO PARA PRODUTO SEM CONTA CONTABIL --
   015 - Lanamento fora do perodo contbil para o financeiro. (BCO, PAG, REC e CNC)
   016 - Lanamento fora do perodo contbil de documentos fiscais de entrada. (DFE)
   017 - Lanamento fora do perodo contbil para documentos fiscais de sada. (DFS)
   */                                                                    

   V_COUNT := 0;
   
   BEGIN
      SELECT IDCONTGERE, IDCONTCOME
         INTO V_CONTRAT, V_CONTCOME
         FROM CONTOPECTBL
        WHERE CDEMPRESA = P_CDEMPRESA;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_CONTRAT := NULL;
         V_CONTCOME := NULL;
   END;

   /*VERIFICACAO DO TOTAL DE DEBITO E CREDITO NO LANCAMENTO CONTABIL*/
   BEGIN
      SELECT NVL(SUM(VRLANCCTBTEMP), 0)
        INTO V_COUNT
        FROM LANCCTBTEMP
       WHERE NRLANCCTBTEMP = P_NRLANCCTBTEMP
         AND CDEMPRESA     = P_CDEMPRESA
      HAVING SUM(VRLANCCTBTEMP) <> 0;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_COUNT := 0;
   END;

   IF V_COUNT <> 0 THEN
      INSERT INTO ERROCTBTEMP
         (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
      VALUES
         (P_CDEMPRESA, P_NRLANCCTBTEMP, '', '001');
      V_COUNT := 0;
   END IF;



   /*VERIFICA SE O LANCAMENTO CONTABIL EST DENTRO DO PERODO CONTBIL*/
   BEGIN
      SELECT COUNT(*)
        INTO V_COUNT
        FROM LANCCTBTEMP L, CONTOPECTBL C
       WHERE L.CDEMPRESA     = P_CDEMPRESA
         AND L.NRLANCCTBTEMP = P_NRLANCCTBTEMP
         AND NOT TRUNC(L.DTLANCCTBTEMP) BETWEEN C.DTINICCONT AND C.DTFINALANC
         AND L.CDEMPRESA = C.CDEMPRESA;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_COUNT := 0;
   END;

    IF V_COUNT <> 0 THEN
      INSERT INTO ERROCTBTEMP
         (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
      VALUES
         (P_CDEMPRESA, P_NRLANCCTBTEMP, '', '002');
    END IF;


    /*VERIFICA SE EXISTE ALGUMA CONTA CONTBIL QUE NO ESTEJA CADASTRADA*/
    FOR C_CONTA IN
       (SELECT NRSEQCTBTEMP
          FROM LANCCTBTEMP L
         WHERE L.CDEMPRESA = P_CDEMPRESA
           AND L.NRLANCCTBTEMP = P_NRLANCCTBTEMP
           AND NOT EXISTS (SELECT 1
                             FROM CONTCTBL C
                            WHERE C.CDCONTCTBL = L.CDCONTCTBL)) LOOP

       INSERT INTO ERROCTBTEMP
          (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
       VALUES
          (P_CDEMPRESA, P_NRLANCCTBTEMP, C_CONTA.NRSEQCTBTEMP, '003');
    END LOOP;

    /*VERIFICA SE EXISTE ALGUM HISTRICO PADRO NO CADASTRADO*/
    SELECT COUNT(*)
      INTO V_COUNT
      FROM LANCCTBTEMP L
     WHERE L.CDEMPRESA = P_CDEMPRESA
       AND L.NRLANCCTBTEMP = P_NRLANCCTBTEMP
       AND NOT EXISTS (SELECT 1
                         FROM HISTPADR H
                        WHERE H.CDHISTPADR = L.CDHISTPADR);

    IF V_COUNT <> 0 THEN
       INSERT INTO ERROCTBTEMP
          (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
       VALUES
          (P_CDEMPRESA, P_NRLANCCTBTEMP, '', '004');
    END IF;

    IF V_CONTCOME = 'S' THEN
    /*VERIFICAO DE EXISTE ALGUM CDIGO COMERCIAL QUE NO ESTEJA CADASTRADO*/
       FOR C_CODCOME IN
          (SELECT IDLANCCTBCOME ID, CDLANCCTBCOME CODIGO, NRSEQCTBTEMP
             FROM LANCCTBTEMP
            WHERE CDEMPRESA = P_CDEMPRESA
              AND NRLANCCTBTEMP = P_NRLANCCTBTEMP
              AND CDLANCCTBCOME IS NOT NULL
              AND IDLANCCTBCOME <> 'N') LOOP

          IF C_CODCOME.ID = 'B' THEN /*BANCO*/
             SELECT COUNT(*)
               INTO V_COUNT
               FROM CONTCORR C
              WHERE C.CDREDUCONT = C_CODCOME.CODIGO;

          ELSIF C_CODCOME.ID = 'F' THEN /*FORNECEDOR*/
             SELECT COUNT(*)
               INTO V_COUNT
               FROM FORNECEDOR
              WHERE CDFORNECED = C_CODCOME.CODIGO;

          ELSE /*CLIENTE*/
              SELECT COUNT(*)
                INTO V_COUNT
                FROM CLIENTE
               WHERE CDCLIENTE = C_CODCOME.CODIGO;
          END IF;

          IF V_COUNT = 0 THEN
             INSERT INTO ERROCTBTEMP
                (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
             VALUES
                (P_CDEMPRESA, P_NRLANCCTBTEMP, C_CODCOME.NRSEQCTBTEMP , '005');
          END IF;
      END LOOP;
   END IF;



   /*VERIFICAO SE EXISTE ALGUMA PARTIDA ZERADA*/
   FOR C_PART IN
      (SELECT NRSEQCTBTEMP
         FROM LANCCTBTEMP
        WHERE CDEMPRESA = P_CDEMPRESA
          AND NRLANCCTBTEMP = P_NRLANCCTBTEMP
          AND VRLANCCTBTEMP = 0) LOOP
      INSERT INTO ERROCTBTEMP
         (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
      VALUES
         (P_CDEMPRESA, P_NRLANCCTBTEMP, C_PART.NRSEQCTBTEMP , '006');
   END LOOP;


   /*VERIFICAO SE EXISTE ALGUM RATEIO ZERADO*/
   IF V_CONTRAT = 'S' THEN
      FOR C_RAT IN
         (SELECT VRRATECTBTEMP, NRSEQCTBTEMP
            FROM RATECTBTEMP
           WHERE CDEMPRESA = P_CDEMPRESA
             AND NRLANCCTBTEMP = P_NRLANCCTBTEMP
             AND VRRATECTBTEMP = 0)LOOP
         INSERT INTO ERROCTBTEMP
            (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
         VALUES
            (P_CDEMPRESA, P_NRLANCCTBTEMP, C_RAT.NRSEQCTBTEMP , '007');
      END LOOP;
   END IF;

   /*VERIFICO SE EXISTE ALGUMA FILIAL NO CADASTRADA*/
   IF V_CONTRAT = 'S' THEN
      FOR C_FIL IN
         (SELECT CDFILIAL, NRSEQCTBTEMP
            FROM RATECTBTEMP
           WHERE CDEMPRESA = P_CDEMPRESA
             AND NRLANCCTBTEMP = P_NRLANCCTBTEMP)LOOP

         SELECT COUNT(*)
           INTO V_COUNT
           FROM FILIAL
          WHERE CDFILIAL = C_FIL.CDFILIAL;

         IF V_COUNT = 0 THEN
            INSERT INTO ERROCTBTEMP
               (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
            VALUES
               (P_CDEMPRESA, P_NRLANCCTBTEMP, C_FIL.NRSEQCTBTEMP , '008');
         END IF;
      END LOOP;
   END IF;

   V_RAT := 'S';
   /*VERIFICAO SE EXISTE ALGUMA FILIAL NO INFORMADA*/
   IF V_CONTRAT = 'S' THEN
      FOR FIL_INF IN
         (SELECT NRSEQCTBTEMP
            FROM LANCCTBTEMP L, CONTCTBL C
           WHERE L.CDEMPRESA = P_CDEMPRESA
             AND L.NRLANCCTBTEMP = P_NRLANCCTBTEMP
             AND L.CDCONTCTBL = C.CDCONTCTBL
             AND C.IDCONTCCUS NOT IN ('L', 'O')
             AND NOT EXISTS (SELECT 1
                              FROM RATECTBTEMP R
                             WHERE R.NRLANCCTBTEMP = L.NRLANCCTBTEMP
                               AND R.NRSEQCTBTEMP = L.NRSEQCTBTEMP))LOOP
            V_RAT := 'N';
            INSERT INTO ERROCTBTEMP
               (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
            VALUES
               (P_CDEMPRESA, P_NRLANCCTBTEMP, FIL_INF.NRSEQCTBTEMP , '012');
      END LOOP;
   END IF;

   IF V_CONTRAT = 'S' THEN
      IF V_RAT = 'S' THEN
         /*VERIFICAO SE EXISTE ALGUM CENTRO DE CUSTO NO CADASTRADO PARA FILIAL INFORMADA*/
         FOR C_CCUST IN
            (SELECT CDFILIAL, CDCENTCUST, NRSEQCTBTEMP
               FROM RATECTBTEMP
              WHERE CDEMPRESA = P_CDEMPRESA
                AND NRLANCCTBTEMP = P_NRLANCCTBTEMP
                AND CDCENTCUST IS NOT NULL
                AND CDFILIAL IS NOT NULL)LOOP

            SELECT COUNT(*)
              INTO V_COUNT
              FROM CENTCUSTFIL
             WHERE CDFILIAL   = C_CCUST.CDFILIAL
               AND CDCENTCUST = C_CCUST.CDCENTCUST;

            IF V_COUNT = 0 THEN
               INSERT INTO ERROCTBTEMP
                  (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
               VALUES
                  (P_CDEMPRESA, P_NRLANCCTBTEMP, C_CCUST.NRSEQCTBTEMP , '009');
            END IF;
         END LOOP;

         /* CENTRO DE CUSTO NO INFORMADO */
         FOR C_CCUST_D IN
            (SELECT CDFILIAL, CDCENTCUST, NRSEQCTBTEMP
               FROM RATECTBTEMP
              WHERE CDEMPRESA = P_CDEMPRESA
                AND NRLANCCTBTEMP = P_NRLANCCTBTEMP
                AND CDCENTCUST IS NULL
                AND CDFILIAL IS NOT NULL)LOOP

               INSERT INTO ERROCTBTEMP
                  (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
               VALUES
                  (P_CDEMPRESA, P_NRLANCCTBTEMP, C_CCUST_D.NRSEQCTBTEMP , '013');
         END LOOP;
      END IF;
   END IF;

   IF V_CONTRAT = 'S' THEN
      /*VERIFICAO SE EXISTE SOMA DE RATEIOS DIFERENTE DO VALOR DA PARTIDA*/
      FOR C_DIF IN
         (SELECT VRLANCCTBTEMP, NRSEQCTBTEMP, NRLANCCTBTEMP, CDEMPRESA
            FROM LANCCTBTEMP
           WHERE CDEMPRESA = P_CDEMPRESA
             AND NRLANCCTBTEMP = P_NRLANCCTBTEMP)LOOP

         SELECT SUM(VRRATECTBTEMP)
           INTO V_COUNT
           FROM RATECTBTEMP
          WHERE CDEMPRESA = P_CDEMPRESA
            AND NRLANCCTBTEMP = P_NRLANCCTBTEMP
            AND NRSEQCTBTEMP  = C_DIF.NRSEQCTBTEMP;

         IF V_COUNT <> C_DIF.VRLANCCTBTEMP THEN
            INSERT INTO ERROCTBTEMP
               (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
            VALUES
               (P_CDEMPRESA, P_NRLANCCTBTEMP, C_DIF.NRSEQCTBTEMP , '010');
         END IF;
      END LOOP;
   END IF;


   IF V_CONTCOME = 'S' THEN
      /*VERIFICA SE EXISTE ALGUMA PARTIDA DE CONTA COMERCIAL SEM CDIGO COMERCIAL*/
      FOR C_CODCOME_NULL IN
          (SELECT NRSEQCTBTEMP
             FROM LANCCTBTEMP
            WHERE CDEMPRESA = P_CDEMPRESA
              AND NRLANCCTBTEMP = P_NRLANCCTBTEMP
              AND IDLANCCTBCOME <> 'N'
              AND CDLANCCTBCOME IS NULL) LOOP

             INSERT INTO ERROCTBTEMP
                (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
             VALUES
                (P_CDEMPRESA, P_NRLANCCTBTEMP, C_CODCOME_NULL.NRSEQCTBTEMP, '011');
      END LOOP;
   END IF;

   /* VERIFICAO SE O LANAMENTO EST DENTRO DOS PERODOS ESPECFICOS PARA CADA MDULO */
   BEGIN
      SELECT DISTINCT IDSISTEMA
        INTO V_IDSISTEMA
        FROM LANCCTBTEMP
       WHERE NRLANCCTBTEMP = P_NRLANCCTBTEMP
         AND CDEMPRESA = P_CDEMPRESA;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_IDSISTEMA := NULL;
   END;

   IF V_IDSISTEMA IS NOT NULL THEN
      IF V_IDSISTEMA IN ('BCO', 'PAG', 'REC', 'CNC') THEN
         BEGIN
            SELECT COUNT(*)
              INTO V_COUNT
              FROM CONTOPECTBL C, LANCCTBTEMP L
             WHERE C.CDEMPRESA = L.CDEMPRESA
               AND L.NRLANCCTBTEMP = P_NRLANCCTBTEMP
               AND L.CDEMPRESA = P_CDEMPRESA
               AND L.DTLANCCTBTEMP BETWEEN C.DTINICCTBL AND C.DTFINACCTBL;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_COUNT := 0;
         END;
         IF V_COUNT = 0 THEN
            INSERT INTO ERROCTBTEMP
                (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
             VALUES
                (P_CDEMPRESA, P_NRLANCCTBTEMP, '', '015');
         END IF;
      ELSIF V_IDSISTEMA = 'DFE' THEN
         BEGIN
            SELECT COUNT(*)
              INTO V_COUNT
              FROM CONTOPECTBL C, LANCCTBTEMP L
             WHERE C.CDEMPRESA = L.CDEMPRESA
               AND L.NRLANCCTBTEMP = P_NRLANCCTBTEMP
               AND L.CDEMPRESA = P_CDEMPRESA
               AND L.DTLANCCTBTEMP BETWEEN C.DTINIINTCONTDFE AND C.DTFININTCONTDFE;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_COUNT := 0;
         END;
         IF V_COUNT = 0 THEN
            INSERT INTO ERROCTBTEMP
                (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
             VALUES
                (P_CDEMPRESA, P_NRLANCCTBTEMP, '', '016');
         END IF;
      ELSIF V_IDSISTEMA = 'DFS' THEN
         BEGIN
            SELECT COUNT(*)
              INTO V_COUNT
              FROM CONTOPECTBL C, LANCCTBTEMP L
             WHERE C.CDEMPRESA = L.CDEMPRESA
               AND L.NRLANCCTBTEMP = P_NRLANCCTBTEMP
               AND L.CDEMPRESA = P_CDEMPRESA
               AND L.DTLANCCTBTEMP BETWEEN C.DTINIINTCONTDFS AND C.DTFININTCONTDFS;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_COUNT := 0;
         END;
         IF V_COUNT = 0 THEN
            INSERT INTO ERROCTBTEMP
                (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
             VALUES
                (P_CDEMPRESA, P_NRLANCCTBTEMP, '', '017');
         END IF;
      END IF;
   END IF;
END;
/


-- VERSAO --
BEGIN ATUVEROBJ('VERIFICA_ERRO_CTB','6.49.009'); END;
/

